@@ -1,21 +1,25 @@
+"""
+@relation(SDOC-SRS-152, scope=file)
+"""
+
+# mypy: disable-error-code="no-untyped-call"
 import os
-from typing import List, NamedTuple, Optional, Tuple
+from typing import List, NamedTuple, Optional, Tuple, Union
 
-import xlrd
-
-from strictdoc.backend.sdoc.models.document import Document
+from strictdoc.backend.excel.import_.excel_sheet_proxy import ExcelSheetProxy
+from strictdoc.backend.sdoc.models.document import SDocDocument
 from strictdoc.backend.sdoc.models.document_config import DocumentConfig
 from strictdoc.backend.sdoc.models.document_grammar import (
     DocumentGrammar,
-    GrammarElement,
 )
+from strictdoc.backend.sdoc.models.grammar_element import (
+    GrammarElement,
+    GrammarElementFieldString,
+)
+from strictdoc.backend.sdoc.models.node import SDocNode, SDocNodeField
 from strictdoc.backend.sdoc.models.object_factory import SDocObjectFactory
 from strictdoc.backend.sdoc.models.reference import ParentReqReference
-from strictdoc.backend.sdoc.models.requirement import (
-    Requirement,
-    RequirementField,
-)
-from strictdoc.backend.sdoc.models.type_system import GrammarElementFieldString
+from strictdoc.helpers.string import ensure_newline
 
 
 class RequirementColumns(NamedTuple):
@@ -27,7 +31,7 @@ class RequirementColumns(NamedTuple):
     extra_header_pairs: List[Tuple[int, str]]
 
 
-def safe_name(dangerous_name):
+def safe_name(dangerous_name: str) -> str:
     dangerous_name = dangerous_name.splitlines()[0]
     dangerous_name = dangerous_name.strip()
     dangerous_name = dangerous_name.upper()
@@ -40,29 +44,34 @@ def safe_name(dangerous_name):
     return dangerous_name
 
 
-# pylint: disable=too-many-instance-attributes
 class ExcelToSDocConverter:
     @staticmethod
-    # optional argument title is present for external scripts to assign a title
-    def convert(excel_file, title=None) -> Document:
-        excel_workbook = xlrd.open_workbook(filename=excel_file, on_demand=True)
-        xlrd_sheet: xlrd.sheet.Sheet = excel_workbook.sheet_by_index(0)
+    def convert(
+        excel_file: str, title: Union[str, None] = None
+    ) -> SDocDocument:
+        """
+        Convert a provided Excel file to an SDoc document.
+
+        Optional argument title is present for external scripts to assign a title.
+        """
+        sheet = ExcelSheetProxy(excel_file)
 
         excel_file_name = os.path.basename(excel_file)
         if title is None:
-            title = excel_file_name + " sheet " + xlrd_sheet.name
+            title = excel_file_name + " sheet " + sheet.name
 
-        all_header_columns = list(range(xlrd_sheet.ncols))
+        all_header_columns = list(range(sheet.ncols))
 
-        for i in range(16):  # the first 16 rows should do ¯\_(ツ)_/¯
-            if xlrd_sheet.row_values(0)[0].strip() != "":
+        # The first 16 rows should do ¯\_(ツ)_/¯.
+        for i in range(16):
+            if sheet.row_values(i)[0].strip() != "":
                 header_row_idx = i
                 break
         else:
             raise NotImplementedError
 
-        header_row = xlrd_sheet.row_values(header_row_idx)
-        safe_header_row = list(safe_name(x) for x in header_row)
+        header_row = sheet.row_values(header_row_idx)
+        safe_header_row = [safe_name(x) for x in header_row]
 
         statement_column = None
         uid_column = None
@@ -107,8 +116,8 @@ class ExcelToSDocConverter:
         document = ExcelToSDocConverter.create_document(
             title, extra_header_pairs
         )
-        for i in range(header_row_idx + 1, xlrd_sheet.nrows):
-            row_values = xlrd_sheet.row_values(i)
+        for i in range(header_row_idx + 1, sheet.nrows):
+            row_values = sheet.row_values(i)
             requirement = ExcelToSDocConverter.create_requirement(
                 row_values, document, columns
             )
@@ -117,23 +126,47 @@ class ExcelToSDocConverter:
         return document
 
     @staticmethod
-    def create_document(title: Optional[str], extra_header_pairs) -> Document:
+    def create_document(
+        title: Optional[str], extra_header_pairs: List[Tuple[int, str]]
+    ) -> SDocDocument:
         document_config = DocumentConfig.default_config(None)
         document_title = title if title else "<No title>"
-        document = Document(document_title, document_config, None, [], [])
+        document = SDocDocument(
+            mid=None,
+            title=document_title,
+            config=document_config,
+            view=None,
+            grammar=None,
+            section_contents=[],
+        )
+
+        # FIXME: This is becoming very limiting. It must work against a complete grammar.
+        fields = list(
+            DocumentGrammar.create_default(document)
+            .elements_by_type["REQUIREMENT"]
+            .fields
+        )
 
-        fields = DocumentGrammar.create_default(document).elements[0].fields
         for _, name in extra_header_pairs:
             fields.extend(
                 [
                     GrammarElementFieldString(
-                        parent=None, title=name, required="False"
+                        parent=None,
+                        title=name,
+                        human_title=None,
+                        required="False",
                     ),
                 ]
             )
 
         requirements_element = GrammarElement(
-            parent=None, tag="REQUIREMENT", fields=fields
+            parent=None,
+            tag="REQUIREMENT",
+            property_is_composite="",
+            property_prefix="",
+            property_view_style="",
+            fields=fields,
+            relations=[],
         )
         elements = [requirements_element]
         grammar = DocumentGrammar(parent=document, elements=elements)
@@ -142,10 +175,11 @@ class ExcelToSDocConverter:
 
     @staticmethod
     def create_requirement(
-        row_values,
-        document,
+        row_values: List[str],
+        document: SDocDocument,
         columns: RequirementColumns,
-    ):
+    ) -> SDocNode:
+        assert columns.statement_column is not None
         statement = row_values[columns.statement_column]
         uid = None
         if columns.uid_column is not None:
@@ -159,55 +193,47 @@ class ExcelToSDocConverter:
             if comment in ("", "-"):
                 comments = None
             else:
-                comments = [comment]
+                comments = [ensure_newline(comment)]
         parent_uid = None
         if columns.parent_column is not None:
             parent_uid = row_values[columns.parent_column].strip()
+            if len(parent_uid) == 0:
+                parent_uid = None
+
         template_requirement = SDocObjectFactory.create_requirement(
             document,
-            requirement_type="REQUIREMENT",
+            node_type="REQUIREMENT",
             title=title,
             uid=uid,
             level=None,
             statement=None,
-            statement_multiline=statement,
+            statement_multiline=ensure_newline(statement),
             rationale=None,
             rationale_multiline=None,
             tags=None,
             comments=comments,
         )
-        if parent_uid is not None:
-            reference = ParentReqReference(template_requirement, parent_uid)
-
-            requirement_field = RequirementField(
-                parent=template_requirement,
-                field_name="REFS",
-                field_value=None,
-                field_value_multiline=None,
-                field_value_references=[reference],
-            )
-            template_requirement.ordered_fields_lookup["REFS"] = [
-                requirement_field
-            ]
-
         for i, name in columns.extra_header_pairs:
             value = row_values[i].strip()
             if value != "":
                 template_requirement.ordered_fields_lookup[name] = [
-                    RequirementField(
+                    SDocNodeField.create_from_string(
                         parent=None,
                         field_name=name,
-                        field_value=None,
-                        field_value_multiline=value,
-                        field_value_references=None,
+                        field_value=ensure_newline(value),
+                        multiline=True,
                     )
                 ]
-
-        requirement = Requirement(
+        if parent_uid is not None:
+            reference = ParentReqReference(
+                template_requirement, parent_uid, role=None
+            )
+            template_requirement.relations = [reference]
+        requirement = SDocNode(
             parent=template_requirement.parent,
-            requirement_type=template_requirement.requirement_type,
+            node_type=template_requirement.node_type,
             fields=list(template_requirement.enumerate_fields()),
+            relations=template_requirement.relations,
         )
-        requirement.ng_level = 1
 
         return requirement
