@@ -1,123 +1,115 @@
-from typing import Optional, List
+# mypy: disable-error-code="arg-type,no-untyped-call"
+from typing import Any, Dict, List, Optional
 
-from strictdoc.backend.sdoc.models.document import Document
-from strictdoc.backend.sdoc.models.requirement import (
-    Requirement,
-    RequirementField,
-)
-from strictdoc.backend.sdoc.models.type_system import RequirementFieldName
+from strictdoc.backend.sdoc.document_reference import DocumentReference
+from strictdoc.backend.sdoc.models.document import SDocDocument
+from strictdoc.backend.sdoc.models.model import RequirementFieldName
+from strictdoc.backend.sdoc.models.node import SDocNode, SDocNodeField
 
 
 class SDocObjectFactory:
     @staticmethod
-    def create_document(title: Optional[str]):
-        return Document(
+    def create_document(title: Optional[str]) -> SDocDocument:
+        return SDocDocument(
+            mid=None,
             title=title if title else "NONAME",
             config=None,
+            view=None,
             grammar=None,
-            free_texts=[],
             section_contents=[],
         )
 
     @staticmethod
-    def create_requirement(  # pylint: disable=too-many-arguments
-        parent,
-        requirement_type: str,
-        uid: Optional[str],
-        level: Optional[str],
-        title: Optional[str],
-        statement: Optional[str],
-        statement_multiline: Optional[str],
-        rationale: Optional[str],
-        rationale_multiline: Optional[str],
-        tags: Optional[str],
-        comments: Optional[List[str]],
-    ) -> Requirement:
-        fields: List[RequirementField] = []
-        if uid:
+    def create_requirement(
+        parent: Any,
+        node_type: Optional[str] = "REQUIREMENT",
+        uid: Optional[str] = None,
+        level: Optional[str] = None,
+        title: Optional[str] = None,
+        statement: Optional[str] = None,
+        statement_multiline: Optional[str] = None,
+        rationale: Optional[str] = None,
+        rationale_multiline: Optional[str] = None,
+        tags: Optional[str] = None,
+        comments: Optional[List[str]] = None,
+    ) -> SDocNode:
+        fields: List[SDocNodeField] = []
+        if uid is not None:
             assert isinstance(uid, str) and len(uid) > 0
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.UID,
                     field_value=uid,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if level:
             assert isinstance(level, str) and len(level) > 0
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.LEVEL,
                     field_value=level,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if title is not None:
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.TITLE,
                     field_value=title,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if statement:
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.STATEMENT,
                     field_value=statement,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if statement_multiline is not None:
-            assert isinstance(
-                statement_multiline, str
-            ), f"{statement_multiline}"
+            assert isinstance(statement_multiline, str), (
+                f"{statement_multiline}"
+            )
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.STATEMENT,
-                    field_value=None,
-                    field_value_multiline=statement_multiline,
-                    field_value_references=None,
+                    field_value=statement_multiline,
+                    multiline=True,
                 )
             )
         if rationale:
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.RATIONALE,
                     field_value=rationale,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if rationale_multiline:
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.RATIONALE,
-                    field_value=None,
-                    field_value_multiline=rationale_multiline,
-                    field_value_references=None,
+                    field_value=rationale_multiline,
+                    multiline=True,
                 )
             )
         if tags is not None:
             assert isinstance(tags, str), f"{tags}"
             fields.append(
-                RequirementField(
+                SDocNodeField.create_from_string(
                     parent=None,
                     field_name=RequirementFieldName.TAGS,
                     field_value=tags,
-                    field_value_multiline=None,
-                    field_value_references=None,
+                    multiline=False,
                 )
             )
         if comments is not None:
@@ -125,20 +117,32 @@ class SDocObjectFactory:
             for comment in comments:
                 assert isinstance(comment, str), f"{comment}"
                 fields.append(
-                    RequirementField(
+                    SDocNodeField.create_from_string(
                         parent=None,
                         field_name=RequirementFieldName.COMMENT,
-                        field_value=None,
-                        field_value_multiline=comment,
-                        field_value_references=None,
+                        field_value=comment,
+                        multiline=True,
                     )
                 )
-        return Requirement(
-            parent=parent, requirement_type=requirement_type, fields=fields
+        requirement = SDocNode(
+            parent=parent,
+            node_type=node_type,
+            fields=fields,
+            relations=[],
         )
+        requirement.ng_document_reference = DocumentReference()
+        if isinstance(parent, SDocDocument):
+            requirement.ng_document_reference.set_document(parent)
+        else:
+            requirement.ng_document_reference.set_document(
+                parent.get_document()
+            )
+        return requirement
 
     @staticmethod
-    def create_requirement_from_dict(requirement_dict, parent, level):
+    def create_requirement_from_dict(
+        requirement_dict: Dict[str, str], parent: Any, level: int
+    ) -> SDocNode:
         assert requirement_dict is not None
         assert parent
         assert isinstance(level, int)
@@ -174,7 +178,7 @@ class SDocObjectFactory:
 
         requirement = SDocObjectFactory.create_requirement(
             parent=parent,
-            requirement_type="REQUIREMENT",
+            node_type="REQUIREMENT",
             uid=uid,
             level=None,
             title=title,
@@ -186,5 +190,4 @@ class SDocObjectFactory:
             comments=None,
         )
 
-        requirement.ng_level = level
         return requirement
