@@ -3,12 +3,12 @@ import strictdoc
 
 class AboutCommand:
     @staticmethod
-    def execute():
+    def execute() -> None:
         print("=============")  # noqa: T201
         print("= StrictDoc =")  # noqa: T201
         print("=============")  # noqa: T201
         print(  # noqa: T201
-            "Purpose: Software for writing technical requirements specifications."  # noqa: E501
+            "Purpose: Software for writing technical requirements specifications."
         )
         print(f"Version: {strictdoc.__version__}")  # noqa: T201
         print(  # noqa: T201
