@@ -3,5 +3,5 @@ import strictdoc
 
 class VersionCommand:
     @staticmethod
-    def execute():
+    def execute() -> None:
         print(strictdoc.__version__)  # noqa: T201
