@@ -1,31 +1,28 @@
+# mypy: disable-error-code="no-untyped-call,union-attr"
 import os
 from pathlib import Path
 
-from strictdoc.backend.sdoc.models.document import Document
-from strictdoc.core.document_tree import DocumentTree
+from strictdoc.backend.sdoc.models.document import SDocDocument
+from strictdoc.core.traceability_index import TraceabilityIndex
 from strictdoc.export.rst.writer import RSTWriter
 
 
-def get_path_components(folder_path):
-    path = os.path.normpath(folder_path)
-    return path.split(os.sep)
-
-
 class DocumentRSTGenerator:
     @staticmethod
     def export_tree(
-        document_tree: DocumentTree, traceability_index, output_rst_root
-    ):
+        traceability_index: TraceabilityIndex, output_rst_root: str
+    ) -> None:
         Path(output_rst_root).mkdir(parents=True, exist_ok=True)
 
-        document: Document
-        for document in document_tree.document_list:
+        document: SDocDocument
+        for document in traceability_index.document_tree.document_list:
             document_content = DocumentRSTGenerator.export(
-                document_tree, document, traceability_index
+                document, traceability_index
             )
 
             output_folder = os.path.join(
-                output_rst_root, document.meta.input_doc_rel_path
+                output_rst_root,
+                document.meta.input_doc_dir_rel_path.relative_path,
             )
             Path(output_folder).mkdir(parents=True, exist_ok=True)
 
@@ -36,10 +33,14 @@ class DocumentRSTGenerator:
                 file.write(document_content)
 
     @staticmethod
-    def export(document_tree: DocumentTree, document, traceability_index):
+    def export(
+        document: SDocDocument, traceability_index: TraceabilityIndex
+    ) -> str:
         writer = RSTWriter(traceability_index)
 
-        single_or_many = len(document_tree.document_list) == 1
+        single_or_many = (
+            len(traceability_index.document_tree.document_list) == 1
+        )
         output = writer.write(document, single_or_many)
 
         return output
